
/*
 * HTApplet.java
 *
 * The MIT License :
 * -----------------
 * Copyright (c) 2005 Embrapa Informtica Agropecuria
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

import java.io.*;
import java.util.*;
import java.util.zip.*;
import java.applet.*;
import java.awt.*;
import java.net.URL;


/**
 * The Demo class implements a demo for HyperTree. It display an HyperTree
 * view of a demo tree.
 * The demo tree is build from a file tree, passed in parameter.
 * Demo could take an argument, the path from which 
 * start the representing of files.
 * If no arguments is given, the treemap start from the root.
 */

public class HTApplet extends Applet { // Appelet

   private static HTFileNode root      = null; // the root of the demo tree
   private static HyperTree  hypertree = null; // the hypertree builded
   private static HTView view; 
   TextField searchField;
   private Image imageH; 
   private Image imageB;
   private Image imageS;
      
    public HTApplet() {		     	
    }

    public void init() {
  
          String pathRoot      = null;
	  String htFile        = getParameter("htfile");	   
	  String htHintColor   = getParameter("hthintcolor");	
	  String htCenterNode  = getParameter("htcenternode");
	  String htBackColor   = getParameter("htbackcolor");
	  String htLinkColor   = getParameter("htlinkcolor");
        String htSearchColor = getParameter("htsearchcolor");
	  String htBorderType  = getParameter("htbordertype");
	  String htFont        = getParameter("htfont");
        String htFontSize    = getParameter("htfontsize");
        String htHomeImage   = getParameter("hthomeimage");
	  String htHelpImage   = getParameter("hthelpimage");
        String htBackImage   = getParameter("htbackimage");
	  String htHelpFile    = getParameter("hthelpfile");
	  String htUrlAdr      = getParameter("hturladr");
	  String htOpenWindow  = getParameter("htopenwindow");
	  String htLeafNode    = getParameter("htleafnode");
 	  String htParentNode  = getParameter("htparentnode");
        String htHasFolder  = getParameter("hthasfolder");




	  if (htFile == null) {
	      System.out.println("Arquivo com a rvore Hiperblica no foi especificado");
		return;
	  }

//	try {
//	      if( Class.forName("netscape.security.PrivilegeManager") != null )
//    			netscape.security.PrivilegeManager.enablePrivilege("UniversalAwtEventQueueAccess");//
//	} catch (Exception e) {
//    		// privilege is not currently granted
//	}

  	  try {
		
            GZIPInputStream zip = new GZIPInputStream( (new URL( getCodeBase(), htFile )).openStream()); 
		ObjectInputStream pi = new ObjectInputStream(zip);
		root = (HTFileNode ) pi.readObject(); 
		zip.close();
 
 	  } catch (Exception e) {
        	System.out.println("Leitura de Objetos  :  " + e.toString());
		return;
        }
     
        if (root == null) {
            System.err.println("Erro : No foi possvel iniciar a rvore Hiperblica");
            return;
        }
 
        hypertree   = new HyperTree(root, this);

        view = hypertree.getView();

	  if (htHintColor!= null)
		view.setHintColor(htHintColor);

	  if (htBackColor!= null)
		view.setBackColor(htBackColor);

	  if (htLinkColor!= null)
		view.setLinkColor(htLinkColor);

	  if (htSearchColor!= null)
		view.setSearchColor(htSearchColor);

	  if (htBorderType != null)
		view.setBorderType(htBorderType);

	  if (htCenterNode != null)
		view.setCenterNode(htCenterNode);

	  if (htHelpFile != null) {
		view.setHelpFile(htHelpFile);
	  }

	  if (htLeafNode  == null) htLeafNode = "Box";

	  if (htLeafNode   != null) {
		view.setLeafNode(htLeafNode);
	  }

	  if (htHasFolder  == null) htHasFolder = "true";

	  if (htHasFolder   != null) {
		view.setHasFolder(htHasFolder);
	  }


	  if (htParentNode  == null) htParentNode = "Box";

	  if (htParentNode   != null) {
		view.setParentNode(htParentNode);
	  }
	  if (htUrlAdr != null)
		view.setUrlAdr(htUrlAdr);

	  if (htFont != null)
		view.setFontNode(htFont);
	  else
		view.setFontNode("Serif");

	  if (htFontSize != null)
		view.setFontSizeNode(((new Integer(htFontSize)).intValue()) );
	  else
		view.setFontSizeNode(12);

	  if (htHomeImage != null) {
		imageH = getImage(getCodeBase(), htHomeImage);
		if (imageH != null) {
			view.setHomeImage(imageH);				
		}
	  }
	  else {
	  }

	  if (htBackImage != null) {
		imageB = getImage(getCodeBase(), htBackImage);
		if (imageB != null) {
			view.setBackImage(imageB);				
		}
	  }
	  else {
	  }

	  if (htOpenWindow != null) {
		view.setOpenWindow(htOpenWindow.toLowerCase());
	  }

	  if (htHelpImage != null) {
		imageS = getImage(getCodeBase(), htHelpImage);
		if (imageS != null) {
			view.setHelpImage(imageS);				
		}
	  }
	  else {
	  }
	

	  // Swing - JPanel 

	  //Container content = getContentPane();
	    
	  //content.setLayout(new BorderLayout());

	  //content.setBackground(Color.gray); 

       //content.setSize(new Dimension(800, 600));

	  //content.add("Center", view); 
	 
	  //content.setVisible(true);

	  // AWT - PAnel

	  searchField = new TextField(10);
        searchField.setBackground(Color.lightGray);
	  searchField.reshape(15, 20, 92, 24);

        view.setTextField(searchField); // **
	  searchField.addTextListener(new HTListenerText(view)); // this)); // **

	  setLayout(new BorderLayout());
	  
	  setBackground(new Color (Color.decode(view.getBackColor()).getRGB())); 
       
        //setSize(new Dimension(800, 600));

        //view.setTextField(searchField); // **
	  //searchField.addTextListener(new HTListenerText(view)); // this)); // **
	  
	  add(searchField);

	  add("Center", view); 	  

	  searchField.reshape(15, 20, 92, 24);
	 
	  setVisible(true);
                     
	  //JFrame viewFrame = new JFrame(root.getName());
          //viewFrame.setContentPane(view);
          //viewFrame.pack();
          //viewFrame.setVisible(true);

    }

   
           
	
/*
    public final void update(Graphics g)
    {
        
            super.paint(g);
    }

    public final void paint(Graphics g)
    {        
           
           super.paint(g);
    }

public void paint(Graphics g) {//conteudo 
	//g.setColor(Color.red); 
	//g.drawString("Eu sou uma applet",25,45); 
	//g.setColor(Color.green); 
	//g.drawString("que desenha um retangulo",45,75); 
	//g.setColor(Color.orange); 
	//g.drawRect(200,50,30,40); 
	view.paintComponent(g);
} 

public void start() {//conteudo 
 	show();
	requestFocus();
}


public void stop() {//conteudo 
	destroy();
}

*/




}

