/*
 * HTAction.java
 * www.bouthier.net
 *
 * The MIT License :
 * -----------------
 * Copyright (c) 2001 Christophe Bouthier
 * This code was modified by Silvio Evangelista e Kleber Sampaio
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
//package hypertree;

import java.awt.*;
import java.awt.event.*;
import java.net.URL;
//import javax.swing.*;
import java.applet.*;
import java.io.*;

/**
 * The HTAction class manage the action on the hypertree :
 * drag of a node...
 *
 * @author     silvio
 * @created    29 de Setembro de 2004
 */

/*
 *  Antes:
 *  class HTAction
 *  extends MouseAdapter
 *  implements MouseMotionListener  {
 */
class HTAction
//extends MouseAdapter
		 implements Serializable, MouseMotionListener, MouseListener, KeyListener {

	private HTDraw model = null;
	// the drawing model
	private HTNodeLabel label = null;

	private HTCoordE startPoint = null;
	// starting point of dragging
	private HTCoordE endPoint = null;
	// ending point of dragging
	private HTCoordS clickPoint = null;
	// clicked point

	private HTDrawNode node;
	private HTDrawNode nodeA;
	private HTDrawNode nodeH = null;
	/**
	 *  Description of the Field
	 */
	protected volatile int fClickCount;
	/**
	 *  Description of the Field
	 */
	protected volatile String nodeID = "";
	/**
	 *  Description of the Field
	 */
	protected volatile String nodeID_A = "";

	private Applet applet;

	private static boolean isDragged = false;


	/*
	 *  --- Constructor ---
	 */
	/**
	 * Constructor.
	 *
	 * @param  model    the drawing model
	 * @param  jApplet  Description of the Parameter
	 */
	HTAction(HTDraw model, Applet jApplet) {
		this.model = model;
		startPoint = new HTCoordE();
		endPoint = new HTCoordE();
		clickPoint = new HTCoordS();
		applet = jApplet;
	}


	/*
	 *  --- MouseAdapter ---
	 */
	/**
	 * Called when a user pressed the mouse button
	 * on the hyperbolic tree.
	 * Used to get the starting point of the drag.
	 *
	 * @param  e  the MouseEvent generated when clicking
	 */
	public void mousePressed(MouseEvent e) {
		if (e.isShiftDown()) {
			model.fastMode(true);
		}
		model.setHint(false);
		isDragged = true;

		startPoint.projectionStoE(e.getX(), e.getY(), model);
	}


	/**
	 * Called when a user release the mouse button
	 * on the hyperbolic tree.
	 * Used to signal the end of the translation.
	 *
	 * @param  e  not used here
	 */
	public void mouseReleased(MouseEvent e) {
		model.fastMode(false);
		model.endTranslation();
		isDragged = false;
	}


	/**
	 * Called when a user move mouse over a node on the hyperbolic tree.
	 * Used to put a hint on the corresponding node (if any) at the
	 * bellowf of the node.
	 *
	 * @param  e  the MouseEvent generated when clicking
	 */

	public void mouseEntered(MouseEvent e) {

	}


	/**
	 * Called when a user move mouse goes off a node on the hyperbolic tree.
	 * Used to drop a hint on the corresponding node (if any).
	 *
	 * @param  e  the MouseEvent generated when clicking
	 */

	public void mouseExited(MouseEvent e) {

	}


	/**
	 * Called when a user clicked on the hyperbolic tree.
	 * Used to put the corresponding node (if any) at the
	 * center of the hyperbolic tree.
	 *
	 * @param  e  the MouseEvent generated when clicking
	 */
	public void mouseClicked(MouseEvent e) {

		model.setHint(false);

		if (e.isShiftDown()) {
			model.restore();
		} else {

			clickPoint.x = e.getX();
			clickPoint.y = e.getY();

			node = model.findNode(clickPoint);

			// 13/09/2004

			int res1 = 0;

			if (node != null) {
				res1 = node.verifyFolder(clickPoint);
			}

			if (res1 == 1) {
				String hasFolder = node.getNodeClientData();

				if (!node.isLeafNode() && node.getHasFolder()) {
					if (hasFolder.equals("11")) {
						node.setNodeClientData("10");
					}

					if (hasFolder.equals("10")) {
						node.setNodeClientData("11");
					}
				}
			}

			// END

			if (node == null) {
				int res = model.verifyHomeClick(clickPoint);

				if (res == 1) {
					// Asking for Help
					String help = model.getHelpFile();

					if (help.length() > 2) {
						boolean UrlOnError = false;
						URL userUrl = null;
						try {
							userUrl = new URL(help);
						} catch (Exception urlError) {
							UrlOnError = true;
						}
						if (!UrlOnError) {
							applet.getAppletContext().showDocument(userUrl, "_blank");
						}
					}
				}
			}

			if ((fClickCount = e.getClickCount()) == 1) {
				new Thread(
					new Runnable() {
						public void run() {
							try {
								Thread.sleep(500);
								if (fClickCount == 1) {

									fClickCount = 0;
									if (node != null) {

										// Silvio em 28/09/2004
										// Vou verificar se o n  do tipo link escondido

										int posH = node.getID().indexOf("_H_");
										if (posH >= 0) {
											HTDrawNode nodeC;
											String idNode = node.getID().substring(0, posH);

											nodeC = model.findNode(idNode);
											if (nodeC != null) {
												model.executaSearchWord(node.getID());//idNode);																					
												model.translateToOrigin(nodeC);
												nodeH = node;
											}
										} else {

											posH = -1;

											if (nodeH != null) {
												posH = nodeH.getID().indexOf("_H_");
											}

											if (posH >= 0) {
												String idNode = nodeH.getID().substring(0, posH);
												if (node.getID().compareTo(idNode) == 0) {
													model.translateToOrigin(nodeH);
												} else {
													model.translateToOrigin(node);
												}

												nodeH = null;
												model.executaSearchWord("");

											} else {
												model.translateToOrigin(node);
											}
										}

										// **** SILVIO

									}
								} else {
									if (node != null) {

										boolean UrlOnError = false;
										URL userUrl = null;
										String adr;
										boolean vazia = true;

										adr = node.getURL();

										if ((adr != null) && (adr.trim().length() > 2)) {
											vazia = false;
										}

										String base = model.getUrlAdr();

										if ((base.length() > 2) && (adr.toLowerCase().indexOf("http:") < 0)) {
											if ((!vazia) && (adr.charAt(0) != '/')) {
												adr = base + "/" + adr;
											} else {
												adr = base + adr;
											}
										}

										try {
											userUrl = new URL(adr);
										} catch (Exception urlError) {
											UrlOnError = true;
										}

										if ((!UrlOnError) && (!vazia)) {
											if (model.getOpenWindow().trim().equals("self")) {
												applet.getAppletContext().showDocument(userUrl, "_parent");
											} else {
												applet.getAppletContext().showDocument(userUrl, "_blank");
											}

										} else {
											System.out.println("URL Invlida " + adr);
										}

										fClickCount = 0;
									}
								}
							} catch (Exception ex) {} finally {}
						}
					}).start();
			}
		}

	}


	/*
	 *  --- MouseMotionListener ---
	 */
	/**
	 * Called when a used drag the mouse on the hyperbolic tree.
	 * Used to translate the hypertree, thus moving the focus.
	 *
	 * @param  e  the MouseEvent generated when draging
	 */
	public void mouseDragged(MouseEvent e) {
		model.setHint(false);
		model.fastMode(true);
		isDragged = true;

		if (startPoint.isValid()) {
			endPoint.projectionStoE(e.getX(), e.getY(), model);
			if (endPoint.isValid()) {
				model.translate(startPoint, endPoint);
			}
		}
	}


	/**
	 * Called when the mouse mouve into the hyperbolic tree.
	 * Not used here.
	 *
	 * @param  e  the MouseEvent generated when mouving
	 */
	public void mouseMoved(MouseEvent e) {

		Point pt = e.getPoint();

		clickPoint.x = pt.x;
		clickPoint.y = pt.y;

		if ((fClickCount > 0) || (isDragged)) {
			return;
		}

		if (node == null) {
			nodeA = null;
		} else {
			if (nodeID.equals(nodeID_A) == false) {
				if (nodeA != null) {
					nodeA.setHasHint(false);
					model.setHint(false);
					model.reDraw();
				}
				nodeA = node;
				nodeID_A = nodeID;
			}
		}

		node = model.findNode(clickPoint);

		if (node != null) {
			if (nodeA != null) {
				nodeA.setEspecialMark(false);
			}

			node.setEspecialMark(true);

			if ((nodeID.equals(nodeID_A)) || (nodeID_A.equals(""))) {
				nodeID_A = nodeID;
			} else {
				nodeID_A = "";

			}
			nodeID = node.getID();

		} else {
			if (nodeA != null) {
				nodeA.setEspecialMark(false);
				nodeA.setHasHint(false);
				model.setHint(false);
				model.reDraw();
			}

			nodeID_A = nodeID;
			nodeID = "";

		}

		if (node != null) {
			new Thread(
				new Runnable() {
					public void run() {
						try {
							if ((nodeID_A.equals("")) || (nodeID.equals(nodeID_A))) {
								Thread.sleep(600);
								if ((fClickCount > 0) || (isDragged)) {
									node.setHasHint(false);
									model.setHint(false);
									return;
								}
								if ((nodeID.equals("") == false) && (nodeID.equals(nodeID_A))) {
									node.setHasHint(true);
									model.setHint(true);
								}
							}
							model.reDraw();
						} catch (Exception ex) {} finally {}
					}
				}).start();
		}
	}


	/**
	 *  Description of the Method
	 *
	 * @param  ev  Description of the Parameter
	 */
	public void keyTyped(KeyEvent ev) {
		//System.out.println("key typed: " + ev.getKeyChar());
	}


	/**
	 *  Description of the Method
	 *
	 * @param  ev  Description of the Parameter
	 */
	public void keyPressed(KeyEvent ev) {
		/*
		 *  int code = ev.getKeyCode();
		 *  switch (code) {
		 *  case KeyEvent.VK_UP:
		 *  System.out.println("Up key");
		 *  break;
		 *  case KeyEvent.VK_DOWN:
		 *  System.out.println("Down key");
		 *  break;
		 *  case KeyEvent.VK_LEFT:
		 *  System.out.println("Left key");
		 *  break;
		 *  case KeyEvent.VK_RIGHT:
		 *  System.out.println("Right key");
		 *  break;
		 *  case KeyEvent.VK_ALT:
		 *  System.out.println("1.) ALT com a tecla "+ ev.getKeyChar());
		 *  break;
		 *  case KeyEvent.VK_CONTROL:
		 *  System.out.println("CONTROL");
		 *  break;
		 *  default: System.out.println("Key press ignored");
		 *  }
		 *  if (ev.isAltDown())
		 *  System.out.println("Key Pressed "+ ev.getKeyChar());
		 */
	}


	/**
	 *  Description of the Method
	 *
	 * @param  ev  Description of the Parameter
	 */
	public void keyReleased(KeyEvent ev) {

		int code = ev.getKeyCode();
		switch (code) {
						case KeyEvent.VK_ESCAPE:

							if (nodeH != null) {
								model.translateToOrigin(nodeH);
								nodeH = null;
								model.executaSearchWord("");
							}

							break;
						/*
						 *  case KeyEvent.VK_DOWN:
						 *  System.out.println("Down key");
						 *  break;
						 *  case KeyEvent.VK_LEFT:
						 *  System.out.println("Left key");
						 *  break;
						 *  case KeyEvent.VK_RIGHT:
						 *  System.out.println("Right key");
						 *  break;
						 *  case KeyEvent.VK_ALT:
						 *  System.out.println("1.) ALT com a tecla "+ ev.getKeyChar());
						 *  break;
						 *  case KeyEvent.VK_CONTROL:
						 *  System.out.println("CONTROL");
						 *  break;
						 *  default: System.out.println("Key press ignored");
						 */
		}
		if (ev.isAltDown()) {
			System.out.println("Key Pressed " + ev.getKeyChar());
		}
	}

}

