/*
 * HTAction.java
 * www.bouthier.net
 *
 * The MIT License :
 * -----------------
 * Copyright (c) 2001 Christophe Bouthier
 * This code was modified by Silvio Evangelista e Kleber Sampaio
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

//package hypertree;

import java.awt.*;

//import javax.swing.*;
import java.applet.*;
import java.util.*;



/**
 * The HTView class implements a view of the HyperTree.
 */
public class HTView
    extends  HTDoubleBuffer { // JPanel or Panel

    private HTModel    model  = null; // the tree model represented
    private HTDraw     draw   = null; // the drawing model
    private HTAction   action = null; // action manager
    private boolean    hasHint= false;
    private TextField  textField;
    private String     vString[] = new String[16];
    private int nAnds = 0;

    private boolean    isSearch    = false;
    private String     hintColor   = "0xF0FFF0";
    private String     centerNode  = null;
    private String     backColor   = "0xDDDDDD";
    private String     linkColor   = "0x000000";
    private String     searchColor = "0xFF0000";
    private String     borderType  = "normal";
    private String     helpFile  = "";
    private String     urlAdr    = "";
    private String     openWindow = "new";
    private String     leafNode   = "Box";
    private String     parentNode  = "Box";
    private boolean    hasFolder   = true;


    private String     fontNode    = "serif";
    private int        fontSize    = 12;
    private boolean    isFirstTime = true;
    private Image      imageH;
    private Image      imageB;
    private Image      imageS;
    private Rectangle  rect; 


  /* --- Constructor --- */

    /**
     * Constructor.
     *
     * @param model    the tree model to view
     */
    public HTView(HTModel model, Applet jApplet) {
        //super(new BorderLayout());
  	  //setLayout(null);
        //setLayout(new BorderLayout());
        //setSize(new Dimension(800, 600));
        //validate();
        //setBackground(Color.white);
        //super(new BorderLayout());
        //setPreferredSize(new Dimension(250, 250));
        
        this.model = model; 
        draw       = new HTDraw(model, this);
	  action     = new HTAction(draw, jApplet);        
        startMouseListening();
	  	 
    }


  /* --- Paint --- */

  
    /**
     * Paint the component.
     *
     * @param g    the graphic context
     */

    public void paintBuffer(Graphics g) {  //paint
	  //super.paint(g);	
        
	  //stopMouseListening();

	  int x, y;

        x = 15;
        y = 50;

	  if (imageB != null) {
        	g.drawImage(imageB, (getSize().width/2)-imageB.getWidth(this)/2, (getSize().height/2)-imageB.getHeight(this)/2, this);            
	  }

        draw.refreshScreenCoordinates();

        draw.drawBranches(g);

        draw.drawNodes(g);

	  if (imageH != null) {
		rect = new Rectangle(x, y, imageH.getWidth(this), imageH.getHeight(this));
        	g.drawImage(imageH, rect.x, rect.y, this);
		if (getCenterNode() != null) {
			g.drawImage(imageH, rect.x+rect.width, rect.y, this);					
		}
	  }

	  if (imageS != null) {	
		if (getCenterNode() != null) 	
        		g.drawImage(imageS, (rect.x*3+rect.width), rect.y, this);				
		else
        		g.drawImage(imageS, (rect.x*3+rect.width), rect.y, this);				
	  }

	  if (hasHint) 
	  	draw.drawHint(g);

	  if ((isFirstTime) && (centerNode != null)) {
		isFirstTime = false;
		HTDrawNode nodeC = draw.findNode(centerNode.toLowerCase());
		if (nodeC != null) {		
			draw.translateToOrigin(nodeC);
		}    			  			
		repaint();		
	  }

	
	  
	  //startMouseListening();	  

    }

    public void setLeafNode(String v)
    {
		leafNode = v;		
    }

    public String getLeafNode()
    {
		return leafNode;		
    }

    public void setParentNode(String v)
    {
		parentNode = v;		
    }

    public String getParentNode()
    {
		return parentNode;		
    }


    public void setHelpImage(Image i)
    {
		imageS = i;		
    }


    public void setHomeImage(Image i)
    {
		imageH = i;		
    }

    public void setBackImage(Image i)
    {
		imageB = i;		
    }


    public Rectangle getHomeRect() {
		return rect;
    }

    /**
     * Set the graphics has a hint.
     *
     * @param g    the graphic context
     */


    public void setHasHint(boolean v) {
	 hasHint = v;
    }

    /**
     * Set the hint color of the nodes
     * 
     * @Param       String RGB color ("0xFFFFFF")
     * @return      void
     *              
     */

    void setHintColor(String color) 
    {
		hintColor = color;
    }

    /**
     * Returns the hint color armazened in view class
     * contains thoses given in parameters.
     *
     * @return      String of RGB color
     *              
     */

    String getHintColor() 
    {
		return hintColor;
    }

  /**
     * Set the id of the center node of the tree
     * 
     * @Param       String
     * @return      void
     *              
     */

    void setCenterNode(String nodeID) 
    {
		centerNode = nodeID;
    }

    /**
     * Returns the id of center node armazened in view class
     * contains thoses given in parameters.
     *
     * @return      String of RGB color
     *              
     */

    String getCenterNode() 
    {
		return centerNode;
    }


    /**
     * Set the border type of the nodes
     * 
     * @Param       String
     * @return      void
     *              
     */

    void setBorderType(String bd) 
    {
		borderType = bd;
    }

    void setHelpFile(String v) {
		helpFile = v;
    }

    void setUrlAdr(String v) {
    		urlAdr = v;
    }

    String getHelpFile() {
		return helpFile;
    }

    String getUrlAdr() {
    		return urlAdr;
    }

    boolean getHasFolder() 
    {
		return hasFolder;
    }
    void setHasFolder(String v) 
    {
		if (v.trim().toUpperCase().indexOf("TRUE")>=0)
			hasFolder = true;
		else
			hasFolder = false;
     }


    /**
     * Returns the border type of the node
     * 
     *
     * @return      String
     *              
     */

    String getBorderType() 
    {
		return borderType;
    }


    /**
     * Set the back color of the nodes
     * 
     * @Param       String RGB color ("0xFFFFFF")
     * @return      void
     *              
     */

    void setBackColor(String color) 
    {
		backColor = color;
    }

    /**
     * Returns the hint color armazened in view class
     * contains thoses given in parameters.
     *
     * @return      String of RGB color
     *              
     */

    String getBackColor() 
    {
		return backColor;
    }

    /**
     * Set the link color of the nodes
     * 
     * @Param       String RGB color ("0xFFFFFF")
     * @return      void
     *              
     */

    void setLinkColor(String color) 
    {
		linkColor = color;
    }

    /**
     * Returns the hint color armazened in view class
     * contains thoses given in parameters.
     *
     * @return      String of RGB color
     *              
     */

    String getLinkColor() 
    {
		return linkColor;
    }


    void setOpenWindow(String v)
    {

		openWindow = v;
    }

    String getOpenWindow()
    {
		return openWindow;
    }

    /**
     * Set the search color of the nodes
     * 
     * @Param       String RGB color ("0xFFFFFF")
     * @return      void
     *              
     */

    void setSearchColor(String color) 
    {
		searchColor = color;
    }

    /**
     * Returns the hint color armazened in view class
     * contains thoses given in parameters.
     *
     * @return      String of RGB color
     *              
     */

    String getSearchColor() 
    {
		return searchColor;
    }

    /**
     * Set the name of the font armazened in view class
     * contains thoses given in parameters.
     *
     * @return      void
     * @param       int
     *              
     */
    
    void setFontNode(String font) 
    { 
		fontNode = font;
    }

   /**
     * Returns the name of the font armazened in view class
     * contains thoses given in parameters.
     *
     * @return      String
     *              
     */

    String getFontNode() {
		return fontNode;
    }

   /**
     * Set size of the font armazened in view class
     * contains thoses given in parameters.
     *
     * @return      void
     * @param       int             
     */

    void setFontSizeNode(int value)
    {
		fontSize = value;
    }

   /**
     * Returns the size of the font armazened in view class
     * contains thoses given in parameters.
     *
     * @return      int
     *              
     */

    int getFontSizeNode() {
		return fontSize;
    }



    /* --- Thread-safe locking --- */
  
    /**
     * Stops the listening of mouse events.
     */
    void stopMouseListening() {
        this.removeMouseListener(action);
        this.removeMouseMotionListener(action);
	  this.removeKeyListener(action); 

    }
    
    /**
     * Starts the listening of mouse events.
     */
    void startMouseListening() {
        this.addMouseListener(action);
        this.addMouseMotionListener(action);
        this.addKeyListener(action); 

    }
  
   /**
     * Set the value of text field (finds field)
     */

    void setTextField(TextField t) {
	textField = t;
    }

   /**
     * find the string S on all node of the tree
     */

    public void searchText(HTNode node, String s) {

		String s1 = " " + s;

		boolean found = false;
		int id;

		node.setHasArc(false);
		node.setHasData(false);

		parserToken(s);

		int posH = s.toUpperCase().indexOf("_H_");
		String s2="";
		if (posH >= 0)
			s2 = s.substring(0, posH);
		else
			s2 = s;

		if (posH > 0)
			if ( (isSearch) && node.getID().toUpperCase().startsWith(s2.toUpperCase()))
				found = true;

		if (posH < 0) {
			if ( (isSearch) && searchToken(node.getName()))
				found = true;

			if ( (isSearch) &&  (!found) && searchToken(node.getHint()))
				found = true;
		}


		//if ( (isSearch) &&  (!found) && ((id=node.getURL().toLowerCase().indexOf(s))>=0))
		//		found = true;
					
		if (found) {
			node.setHasData(true);
			//System.out.println(node.getName() + " " + node.getHasData());
		}
		
		for (Enumeration e = node.childrenNode(); e.hasMoreElements(); ) {
            	HTNode childNode = (HTNode) e.nextElement();

			childNode.setHasData(false);
			childNode.setHasArc(false);

			if (childNode != null) {
            		if (!childNode.isLeafNode()) {				
                			searchText(childNode, s);
					if ( (childNode.getHasData()) || ( childNode.getHasArc()))	
						node.setHasArc(true);					
            		}
				else {
					found = false;

					posH = s.toUpperCase().indexOf("_H_");
					s2="";
					if (posH >= 0)
						s2 = s.substring(0, posH);
					else
						s2 = s;
					if (posH > 0)
						if ( (isSearch) && childNode.getID().toUpperCase().startsWith(s2.toUpperCase())) {							
							found = true;
					}

					if (posH < 0) {
						if ( (isSearch) && searchToken(childNode.getName()))
							found = true;

						if ( (isSearch) &&  (!found) && searchToken(childNode.getHint()) )
							found = true;
					}


					//if ((isSearch) && ((id=childNode.getName().toLowerCase().indexOf(s))>=0))						
					//		found = true;

					//if ((isSearch) && (!found) && ((id=childNode.getHint().toLowerCase().indexOf(s))>=0))
					//		found = true;

					//if ((isSearch) && (!found) && ((id=childNode.getURL().toLowerCase().indexOf(s))>=0))
					//		found = true;
					
					if (found) {
						childNode.setHasData(true);
						node.setHasArc(true);						
					}
				}
													
			}
		}

	}

  // Silvio: 29/09/2004

    void executaSearchWord(String search) {

		if ((search == null) || (search.length() == 0))
			isSearch = false;
		else
			isSearch = true;
			
		HTModelNode nodeD = model.getRoot();

		searchText(nodeD.getHTnode(), search.toLowerCase());

		repaint();

		
    }
    // *** Silvio


	boolean searchToken(String v) {
		String s1, s;
		int nFound=0;
		int id;
		v = v.toLowerCase();

		for(int i=0; i < nAnds;i++) {
			s = vString[i];
			s1 = " " + s.trim();

			if ( (isSearch) && ( ((id=v.indexOf(s1))>=0) || 
                   	     (v.startsWith(s)) ))			
				nFound++;
		}
		if (nFound >= nAnds) 
			return(true);
		else
			return(false);


	}

	void parserToken(String v) {
		String s="";

		int j=0;


		if (v != null) {
			for(j=0; j < v.length(); j++) if (v.charAt(j) != ' ') break;

			nAnds =0;
			for (int i=j; i < v.length();) {		
				if (v.charAt(i) == '&') { // Antes tinha &
					if (s.length() >= 1) {
						vString[nAnds] = s;
						nAnds++;
						for(j=i+1; j < v.length(); j++) if (v.charAt(j) != ' ') break;
						i=j-1;
					}
					s = "";
				}
				else {					
					s = s + v.charAt(i);
				}
				i++;								
			}
			vString[nAnds] = s;
			nAnds++;	
		}		

	}
   
    String getTextField() {
		String search = textField.getText();

		if ((search == null) || (search.length() == 0))
			isSearch = false;
		else
			isSearch = true;
			
		HTModelNode nodeD = model.getRoot();

		searchText(nodeD.getHTnode(), search.toLowerCase());

		if (search.length() == 1)
			draw.translateToRootOrigin();

		repaint();

		return search;
    }

    String getSearchWord() {
		return (textField.getText());
    }


    int getNumAnds() {
	return(nAnds);
    }
  
    String getWordI(int i) {
	if (i < nAnds) 
		return (vString[i]);
	else
		return (null);
    }



}

